﻿/**
* DevExpress Dashboard (_model-subscriber.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import * as ko from 'knockout';
import { PropertyCategory } from '../../model/metadata/_base-metadata';
export class ModelSubscriber {
    static changePropertyQuietly(property, func) {
        try {
            property[ModelSubscriber.dxSubscriptionSuspend] = true;
            func();
        }
        finally {
            delete property[ModelSubscriber.dxSubscriptionSuspend];
        }
    }
    constructor(_model) {
        this._model = _model;
        this.handlers = [];
        this._subscribe(_model);
    }
    _unsubscribe(model) {
        if (model.disposed)
            return;
        var serializationsInfo = model.getInfo();
        serializationsInfo
            .filter(serializationInfo => !!serializationInfo.modelName)
            .forEach(serializationInfo => {
            var property = model[serializationInfo.propertyName];
            var propertyValue = ko.unwrap(property);
            if (property && property[ModelSubscriber.dxSubscription]) {
                var subscription = property[ModelSubscriber.dxSubscription];
                subscription.dispose();
                delete property[ModelSubscriber.dxSubscription];
            }
            if (this._isPropertySerializeModel(propertyValue)) {
                this._unsubscribe(propertyValue);
            }
        });
    }
    _addDisposable(model, subscription, property) {
        if (model.addDisposable)
            model.addDisposable(subscription);
        else
            property[ModelSubscriber.dxSubscription] = subscription;
    }
    _subscribe(model) {
        if (!model.getInfo) {
            return;
        }
        var serializationsInfo = model.getInfo();
        serializationsInfo
            .filter(serializationInfo => !!serializationInfo.modelName)
            .forEach(serializationInfo => {
            var property = model[serializationInfo.propertyName];
            var category = !!serializationInfo['category'] ? serializationInfo['category'] : PropertyCategory.Data;
            var propertyValue = ko.unwrap(property);
            if (Array.isArray(property)) {
                throw new Error('Non-observable arrays are not supported.');
            }
            if (category === PropertyCategory.NoUpdate)
                return;
            if (Array.isArray(propertyValue) && !property[ModelSubscriber.dxSubscription]) {
                propertyValue.forEach(item => this._subscribe(item));
                const subscription = property.subscribe((arrayChanges) => {
                    arrayChanges.forEach(arrayChange => {
                        var changedStatus = 'unknown';
                        if (arrayChange.status === 'added') {
                            this._subscribe(arrayChange.value);
                            changedStatus = 'added';
                        }
                        if (arrayChange.status === 'deleted') {
                            this._unsubscribe(arrayChange.value);
                            changedStatus = 'deleted';
                        }
                        if (property[ModelSubscriber.dxSubscriptionSuspend] !== true)
                            this._propertyChanged(category, arrayChange.value, serializationInfo.propertyName, changedStatus);
                    });
                }, null, 'arrayChange');
                this._addDisposable(model, subscription, property);
            }
            else if (ko.isObservable(property) && !property[ModelSubscriber.dxSubscription]) {
                var getSubscribeHandler = prevModel => model => {
                    if (this._isPropertySerializeModel(prevModel)) {
                        this._unsubscribe(prevModel);
                    }
                    if (this._isPropertySerializeModel(model)) {
                        this._subscribe(model);
                    }
                };
                var subscriberHandler = getSubscribeHandler(propertyValue);
                const subscription = property.subscribe(val => {
                    if (property[ModelSubscriber.dxSubscriptionSuspend] !== true) {
                        this._propertyChanged(category, model, serializationInfo.propertyName, 'changed');
                    }
                    if (category !== PropertyCategory.NoUpdateByObservableValue) {
                        subscriberHandler(val);
                        subscriberHandler = getSubscribeHandler(val);
                    }
                });
                this._addDisposable(model, subscription, property);
                if (category !== PropertyCategory.NoUpdateByObservableValue && this._isPropertySerializeModel(propertyValue)) {
                    this._subscribe(propertyValue);
                }
            }
            else if (!ko.isObservable(property) && this._isPropertySerializeModel(propertyValue)) {
                this._subscribe(propertyValue);
            }
        });
    }
    _propertyChanged(category, model, propertyName, status) {
        this.handlers.forEach(handler => handler(category, model, propertyName, status));
    }
    registerHandler(handler) {
        this.handlers.push(handler);
    }
    _isPropertySerializeModel(propertyValue) {
        return propertyValue && propertyValue['getInfo'];
    }
    dispose() {
        this._unsubscribe(this._model);
    }
}
ModelSubscriber.dxSubscription = '__dx_notifier_subscription';
ModelSubscriber.dxSubscriptionSuspend = '__dx_notifier_subscription_suspend';
