﻿/**
* DevExpress Dashboard (_formatter.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { format as stringFormat } from 'devextreme/core/utils/string';
import { DashboardPrivateSettings } from '../viewer-parts/legacy-settings';
import { DashboardFormatHelper, invariantCurrencyIdentifier } from './_format-helper';
import { localizer } from './_localizer';
import { type } from './_utils';
import { getLocalizationById } from './localization/_default';
export let defaultNumericFormat = {
    format: 'fixedPoint',
    unitPower: 'auto',
    precision: 0,
    significantDigits: 3
};
export let defaultPercentFormat = {
    format: 'percent',
    unitPower: 0,
    precision: 2,
    significantDigits: 0,
    showTrailingZeros: false
};
export let defaultScientificFormat = {
    format: 'exponential',
    precision: 2
};
export let _types = {
    Abbreviated: 'abbr',
    Full: 'full',
    Long: 'long',
    Numeric: 'num',
    Short: 'short',
    TimeOnly: 'timeOnly'
};
export function format(value, formatViewModel) {
    var str = localizer.getPredefinedString(value);
    if (!type.isDefined(str)) {
        var numericFormat = formatViewModel && formatViewModel.NumericFormat, dateTimeFormat = formatViewModel && formatViewModel.DateTimeFormat;
        if (numericFormat) {
            str = formatNumeric(value, numericFormat);
        }
        else {
            str = !dateTimeFormat ? formatObject(value) : formatDateTime(value, dateTimeFormat);
        }
    }
    return str;
}
export function formatByFormatInfo(value, formatInfo) {
    var str = localizer.getPredefinedString(value);
    if (!type.isDefined(str)) {
        return DashboardFormatHelper.format(value, formatInfo);
    }
    return str;
}
export function formatFilterValue(filterValue) {
    if (filterValue.Value !== undefined) {
        return format(filterValue.Value, filterValue.Format);
    }
    let rangeLeft;
    let rangeRight;
    if (type.isDefined(filterValue.RangeLeft)) {
        rangeLeft = format(filterValue.RangeLeft, filterValue.Format);
    }
    if (type.isDefined(filterValue.RangeRight)) {
        rangeRight = format(filterValue.RangeRight, filterValue.Format);
    }
    return constructIntervalFilterText({ left: rangeLeft, right: rangeRight });
}
export function constructIntervalFilterText(rangeText) {
    let rangePattern = getLocalizationById('DashboardStringId.FromToDatePeriodCaption');
    let fromPattern = getLocalizationById('DashboardStringId.FromDatePeriodCaption');
    let toPattern = getLocalizationById('DashboardStringId.ToDatePeriodCaption');
    if (rangeText.left && rangeText.right) {
        if (rangeText.right != rangeText.left)
            return stringFormat(rangePattern, rangeText.left, rangeText.right);
        else
            return rangeText.left;
    }
    if (rangeText.left) {
        return stringFormat(fromPattern, rangeText.left);
    }
    if (rangeText.right) {
        return stringFormat(toPattern, rangeText.right);
    }
}
export function formatNumeric(value, numericFormatViewModel) {
    if (!numericFormatViewModel) {
        return value.toString();
    }
    else {
        var format = _convertToNumberFormat(numericFormatViewModel);
        return DashboardFormatHelper.format(value, format);
    }
}
export function formatDateTime(value, dateFormatViewModel) {
    var format = _convertToDateFormat(dateFormatViewModel);
    return DashboardFormatHelper.format(value, format);
}
export function formatObject(value) {
    return value == null ? '' : value.toString();
}
export function formatPercentValue(value) {
    return DashboardFormatHelper.format(value, defaultPercentFormat);
}
export function formatScientificAxisValue(value) {
    return DashboardFormatHelper.format(value, defaultScientificFormat);
}
export function formatAxisValue(value, axisMin, axisMax) {
    return DashboardFormatHelper.format(value, getAxisFormat(axisMin, axisMax));
}
export function getAxisFormat(axisMin, axisMax) {
    return {
        format: 'fixedPoint',
        unitPower: calculateUnitPower(axisMin, axisMax),
        precision: calculatePrecision(axisMin, axisMax),
        significantDigits: 0,
        showTrailingZeros: false
    };
}
export function calculateUnitPower(axisMin, axisMax) {
    var range = axisMax - axisMin;
    if (range >= 1000000000)
        return 3;
    if (range >= 1000000)
        return 2;
    if (range >= 1000)
        return 1;
    return 0;
}
export function calculatePrecision(axisMin, axisMax) {
    var precision = 0, range = axisMax - axisMin;
    if (range > 0) {
        var smallValue = Math.pow(10, -precision);
        while (range < smallValue) {
            smallValue /= 10;
            precision++;
        }
    }
    return precision + 2;
}
export function convertToFormat(formatViewModel) {
    if (formatViewModel) {
        let numericFormat = formatViewModel.NumericFormat;
        if (numericFormat)
            return _convertToNumberFormat(numericFormat);
        let dateTimeFormat = formatViewModel.DateTimeFormat;
        if (dateTimeFormat)
            return _convertToDateFormat(dateTimeFormat);
    }
    return null;
}
export function _convertToNumberFormat(numericFormatViewModel) {
    var formatInfo = null, formatType = numericFormatViewModel ? numericFormatViewModel.FormatType : undefined, unit = numericFormatViewModel ? numericFormatViewModel.Unit : undefined;
    if (formatType === 'Custom') {
        if (DashboardPrivateSettings.customNumericFormatMode !== 'Disabled') {
            return {
                type: numericFormatViewModel.CustomFormatString,
                unlimitedIntegerDigits: true
            };
        }
        else {
            formatType = 'Auto';
        }
    }
    if (formatType === 'General') {
        formatInfo = { format: 'general' };
    }
    else {
        formatInfo = {};
        formatInfo.format = _convertNumericFormat(formatType);
        formatInfo.currency = numericFormatViewModel.Currency;
        if (formatInfo.currency === invariantCurrencyIdentifier) {
            console.log('A web server does not support a currency specified for the dashboard. Refer to https://go.devexpress.com/dashboardInvariantCurrency.aspx to learn more.');
        }
        if (numericFormatViewModel.IncludeGroupSeparator) {
            formatInfo.includeGroupSeparator = numericFormatViewModel.IncludeGroupSeparator;
        }
        if (numericFormatViewModel.ForcePlusSign) {
            formatInfo.plus = numericFormatViewModel.ForcePlusSign;
        }
        formatInfo.precision = numericFormatViewModel.Precision;
        if (unit && unit === 'Auto' && formatType !== 'Percent') {
            formatInfo.significantDigits = numericFormatViewModel.SignificantDigits;
        }
        if (unit && formatType === 'Number' || formatType === 'Currency') {
            formatInfo.unitPower = _convertNumericUnit(unit);
        }
        formatInfo.dateType = null;
    }
    return formatInfo;
}
export function _convertToDateFormat(dateFormatViewModel) {
    if (dateFormatViewModel) {
        let weekOptions = {};
        if (type.isDefined(dateFormatViewModel.FirstDayOfWeek))
            weekOptions.firstDayOfWeek = dateFormatViewModel.FirstDayOfWeek;
        if (type.isDefined(dateFormatViewModel.CalendarWeekRule))
            weekOptions.calendarWeekRule = dateFormatViewModel.CalendarWeekRule;
        let syntheticGroupInterval = _getSyntheticDateTimeGroupInterval(dateFormatViewModel.GroupInterval, dateFormatViewModel.ExactDateFormat);
        switch (syntheticGroupInterval) {
            case 'WeekYear':
                return { ...weekOptions,
                    format: 'weekYear',
                    dateType: dateFormatViewModel.DateFormat === 'Long' ? _types.Long : _types.Short
                };
            case 'MonthYear':
                return { format: 'monthYear', dateType: _types.Full };
            case 'QuarterYear':
                return { format: 'quarterYear', dateType: _types.Full };
            case 'DayMonthYear':
                return {
                    format: 'dayMonthYear',
                    dateType: dateFormatViewModel.DateFormat === 'Long' ? _types.Long : _types.Short
                };
            case 'DateHour':
                if (dateFormatViewModel.DateHourFormat === 'Long')
                    return { format: 'dateHour', dateType: _types.Long };
                else
                    return { format: 'dateHour', dateType: dateFormatViewModel.DateHourFormat === 'Short' ? _types.Short : _types.TimeOnly };
            case 'DateHourMinute':
                if (dateFormatViewModel.DateHourMinuteFormat === 'Long')
                    return { format: 'dateHourMinute', dateType: _types.Long };
                else
                    return { format: 'dateHourMinute', dateType: dateFormatViewModel.DateHourMinuteFormat === 'Short' ? _types.Short : _types.TimeOnly };
            case 'DateHourMinuteSecond':
                if (dateFormatViewModel.DateTimeFormat === 'Long')
                    return { format: 'dateHourMinuteSecond', dateType: _types.Long };
                else
                    return { format: 'dateHourMinuteSecond', dateType: dateFormatViewModel.DateTimeFormat === 'Short' ? _types.Short : _types.TimeOnly };
            case 'Year':
                return {
                    format: 'year',
                    dateType: dateFormatViewModel.YearFormat === 'Abbreviated' ? _types.Abbreviated : _types.Full
                };
            case 'DateYear':
                return {
                    format: 'dateYear',
                    dateType: dateFormatViewModel.YearFormat === 'Abbreviated' ? _types.Abbreviated : _types.Full
                };
            case 'Quarter':
                if (dateFormatViewModel.QuarterFormat === 'Numeric')
                    return { format: 'quarter', dateType: _types.Numeric };
                else
                    return { format: 'quarter', dateType: _types.Full };
            case 'Month':
                if (dateFormatViewModel.MonthFormat === 'Numeric')
                    return { format: 'month', dateType: _types.Numeric };
                else
                    return { format: 'month', dateType: dateFormatViewModel.MonthFormat === 'Abbreviated' ? _types.Abbreviated : _types.Full };
            case 'Hour':
                return {
                    format: 'hour',
                    dateType: dateFormatViewModel.HourFormat === 'Long' ? _types.Long : _types.Short
                };
            case 'DayOfWeek':
                if (dateFormatViewModel.DayOfWeekFormat === 'Numeric')
                    return { format: 'dayOfWeek', dateType: _types.Numeric };
                else
                    return { format: 'dayOfWeek', dateType: dateFormatViewModel.DayOfWeekFormat === 'Abbreviated' ? _types.Abbreviated : _types.Full };
            default:
                return { format: dateTimeGroupIntervalToCamelCase[dateFormatViewModel.GroupInterval], dateType: _types.Numeric };
        }
    }
    else {
        return null;
    }
}
export function _getSyntheticDateTimeGroupInterval(groupInterval, exactDateFormat) {
    if (groupInterval != 'None')
        return groupInterval;
    switch (exactDateFormat) {
        case 'Year':
            return 'DateYear';
        case 'Quarter':
            return 'QuarterYear';
        case 'Month':
            return 'MonthYear';
        case 'Day':
            return 'DayMonthYear';
        case 'Hour':
            return 'DateHour';
        case 'Minute':
            return 'DateHourMinute';
        case 'Second':
            return 'DateHourMinuteSecond';
        default:
            return null;
    }
}
export function _convertNumericFormat(formatType) {
    switch (formatType) {
        case 'Number': return 'fixedPoint';
        case 'Currency': return 'currency';
        case 'Scientific': return 'exponential';
        case 'Percent': return 'percent';
        default: return undefined;
    }
}
export function _convertNumericUnit(numericUnit) {
    switch (numericUnit) {
        case 'Auto': return 'auto';
        case 'Thousands': return 1;
        case 'Millions': return 2;
        case 'Billions': return 3;
        default: return 0;
    }
}
let dateTimeGroupIntervalToCamelCase = {
    'Year': 'year',
    'Quarter': 'quarter',
    'Month': 'month',
    'Day': 'day',
    'Hour': 'hour',
    'Minute': 'minute',
    'Second': 'second',
    'DayOfYear': 'dayOfYear',
    'DayOfWeek': 'dayOfWeek',
    'WeekOfYear': 'weekOfYear',
    'WeekOfMonth': 'weekOfMonth',
    'WeekYear': 'weekYear',
    'MonthYear': 'monthYear',
    'QuarterYear': 'quarterYear',
    'DayMonthYear': 'dayMonthYear',
    'DateHour': 'dateHour',
    'DateHourMinute': 'dateHourMinute',
    'DateHourMinuteSecond': 'dateHourMinuteSecond',
    'None': 'none'
};
