﻿/**
* DevExpress Dashboard (date-filter-widget.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import fx from 'devextreme/animation/fx';
import { format as stringFormat } from 'devextreme/core/utils/string';
import dxButton from 'devextreme/ui/button';
import dxButtonGroup from 'devextreme/ui/button_group';
import dxCalendar from 'devextreme/ui/calendar';
import dxDateBox from 'devextreme/ui/date_box';
import dxOverlay from 'devextreme/ui/overlay/ui.overlay';
import { constructIntervalFilterText } from '../../../data/_formatter';
import { $unwrap, getWidth, wrapPublicElement } from '../../../data/_jquery-helpers';
import { getLocalizationById } from '../../../data/localization/_default';
import { CalendarHelper } from '../../_utils';
import { RenderHelper } from '../../widgets/_render-helper';
import { DatePickerButtons, dateFilterWidgetClassNames } from './_date-filter-widget-options';
import { PopupCalendarWidgets } from './_popup-calendar-widgets';
export class DateFilterWidget {
    get calendarFrom() {
        return (this._calendarWidgetsFrom && this._calendarWidgetsFrom.calendarWidget || null);
    }
    get calendarTo() {
        return (this._calendarWidgetsTo && this._calendarWidgetsTo.calendarWidget || null);
    }
    constructor(element, viewerOptions, boundaryElementContainer) {
        this.boundaryElementContainer = boundaryElementContainer;
        this._defaultButtonText = getLocalizationById('DashboardStringId.DateFilterDatePickerButtonDefaultText');
        this._overlayShown = false;
        this._lockSelectionEvents = false;
        this._updateScrollableContainer = () => { };
        this._getBtnsContainerScrollWidth = () => 0;
        this._datePickerContent = new DatePickerButtons();
        this.quickButtons = [];
        this._handleKeyDown = (event) => {
            if (event.key === 'Escape' || event.code === 'Escape') {
                this._removeOverlayEventListener();
                this._overlay?.hide();
            }
        };
        this._parent = document.createElement('div');
        element.appendChild(this._parent);
        this._scrollableContent = RenderHelper.wrapScrollable(this._parent, viewerOptions.overflow, 'both');
        this._updateScrollableContainer = () => RenderHelper.updateScrollable(this._parent);
    }
    element() {
        return wrapPublicElement(this._scrollableContent);
    }
    _update(widgetOptions) {
        this._options = { ...this._getDefaultOptions(), ...widgetOptions };
        this._dispose();
        this._startDate = this._options.startDate;
        this._endDate = this._options.endDate;
        var hasQuickFilters = this._options.buttonsInfo.length > 0;
        var content = this._scrollableContent;
        this._widgetDiv = this._createDiv(content, dateFilterWidgetClassNames.widget);
        var buttonsDiv = this._createButtonsDiv();
        this._getBtnsContainerScrollWidth = () => Math.ceil(buttonsDiv.scrollWidth);
        if (this._options.datePickerLocation == 'Near')
            this._addDatePicker(buttonsDiv);
        if (hasQuickFilters) {
            this._options.buttonsInfo.forEach(info => {
                let buttonDiv = this._createDiv(buttonsDiv, dateFilterWidgetClassNames.button);
                var button = new dxButtonGroup(buttonDiv, {
                    items: [{ text: info }],
                    selectionMode: 'multiple',
                    focusStateEnabled: true,
                    onSelectionChanged: e => {
                        if (!this._lockSelectionEvents) {
                            if (e.removedItems.length > 0)
                                this._options.clearAction();
                            if (e.addedItems.length > 0) {
                                this._selectQuickButton(e.component);
                                this._options.buttonClick(e.component.option('items')[0].text);
                            }
                        }
                    }
                });
                this.quickButtons.push(button);
            });
        }
        if (this._options.datePickerLocation == 'Far')
            this._addDatePicker(buttonsDiv);
        var overlayDiv = this._createDiv(this._widgetDiv, dateFilterWidgetClassNames.overlayWrapper);
        this._createDropDownContent();
        this._overlay = new dxOverlay(overlayDiv, this._getOverlayOptions());
        this._updateDropDownButtonText();
    }
    _getHeight() {
        return this._widgetDiv.offsetHeight;
    }
    _updateSize(width, height) {
        this._widgetDiv.style.height = 'auto';
        if (this._options.arrangementMode === 'Vertical') {
            this._widgetDiv.style.height = height + 'px';
            this._widgetDiv.style.width = width + 'px';
            let btnsContainerWidth = this._getBtnsContainerScrollWidth();
            let negativeBtnsContainerMargin = this._options.hasOuterMargin ? 0 : 6;
            if (btnsContainerWidth - Math.ceil(width) - negativeBtnsContainerMargin > 0) {
                this._widgetDiv.style.width = `${btnsContainerWidth}px`;
            }
            this._updateScrollableContainer();
        }
        else
            this._widgetDiv.style.width = width + 'px';
    }
    _setSelectedValues(values) {
        if (!!values && values.length > 0 && (!!values[0] || !!values[1])) {
            this._startDate = values[0];
            this._endDate = values[1];
            this._setDatePickerBtnState(true);
        }
        else {
            this._clearSelectedPeriods();
            this._setDatePickerBtnState(false);
        }
        this._updateDropDownButtonText();
    }
    _setPeriod(periodIndex) {
        var buttons = this.quickButtons;
        if (periodIndex != null && periodIndex < buttons.length) {
            this._setDatePickerBtnState(false);
            var button = buttons[periodIndex];
            button.option('selectedItems', button.option('items'));
        }
    }
    _clearSelectedPeriods() {
        this._performWithLockedEvents(() => {
            var buttons = this.quickButtons;
            for (var i = 0; i < buttons.length; i++) {
                buttons[i].option('selectedItems', []);
            }
        });
    }
    _clearSelectedValues() {
        this._setSelectedValues([]);
        this._clearSelectedPeriods();
    }
    _submit() {
        var filterType = this._options.filterType;
        if (filterType === 'After') {
            this._startDate = this._calendarWidgetsFrom.calendarWidget.option('value');
            this._endDate = null;
        }
        if (filterType === 'Before') {
            this._startDate = null;
            this._endDate = this._calendarWidgetsTo.calendarWidget.option('value');
        }
        if (filterType === 'Exact') {
            this._startDate = this._calendarWidgetsFrom.calendarWidget.option('value');
            this._endDate = this._calendarWidgetsFrom.calendarWidget.option('value');
        }
        if (filterType === 'Between') {
            this._startDate = this._calendarWidgetsFrom.calendarWidget.option('value');
            this._endDate = this._calendarWidgetsTo.calendarWidget.option('value');
        }
        this._applyValues();
        this._removeOverlayEventListener();
        this._overlay.hide();
        this._updateDropDownButtonText();
    }
    _removeOverlayEventListener() {
        $unwrap(this._overlay.content())?.removeEventListener('keydown', this._handleKeyDown);
    }
    _applyValues() {
        this._processItemClick([this._startDate, this._endDate]);
        this._performWithLockedEvents(() => {
            for (var i = 0; i < this.quickButtons.length; i++) {
                this.quickButtons[i].option('selectedItems', []);
            }
            this._setDatePickerBtnState(true);
        });
    }
    _addDatePicker(parentElement) {
        this._datePickerDiv = this._createDiv(parentElement, dateFilterWidgetClassNames.button);
        this.datePickerButton = new dxButtonGroup(this._datePickerDiv, {
            items: this._datePickerContent.getButtons(),
            focusStateEnabled: true,
            keyExpr: 'key',
            selectionMode: 'multiple',
            onSelectionChanged: e => {
                if (!this._lockSelectionEvents) {
                    if (e.removedItems.length > 0 && e.removedItems[0].key === 'Check') {
                        this._onDatePickerUnchecked();
                    }
                    var clickedButton = e.addedItems[0];
                    if (clickedButton) {
                        if (clickedButton.key === 'DropDown') {
                            this._showPopup();
                        }
                        if (clickedButton.key === 'Check') {
                            if (this._startDate == null && this._endDate == null) {
                                e.component.option('selectedItemKeys', []);
                                this._showPopup();
                            }
                            else {
                                this._applyValues();
                            }
                        }
                        var selection = e.component.option('selectedItemKeys');
                        selection = selection.filter((key) => { return key !== 'DropDown'; });
                        e.component.option('selectedItemKeys', selection);
                    }
                }
            }
        });
    }
    _getStartCalendarValue() {
        return this._startDate || this._options.minimum;
    }
    _getEndCalendarValue() {
        return this._endDate || this._options.maximum;
    }
    _createDiv(container, ...classes) {
        var div = document.createElement('div');
        if (container != null)
            container.appendChild(div);
        classes.forEach(className => this._addClass(div, className));
        return div;
    }
    _createButtonsDiv() {
        var classNames = [dateFilterWidgetClassNames.buttons];
        if (this._options.arrangementMode === 'Vertical')
            classNames.push(dateFilterWidgetClassNames.buttonsTopDown);
        else
            classNames.push(dateFilterWidgetClassNames.buttonsLeftToRight);
        if (!this._options.hasOuterMargin)
            classNames.push(dateFilterWidgetClassNames.buttonsNoPaddings);
        return this._createDiv(this._widgetDiv, ...classNames);
    }
    _createDropDownContent() {
        var filterType = this._options.filterType;
        this._dropDownContentDiv = this._createDiv(null, dateFilterWidgetClassNames.overlay);
        if (this._options.mobileLayout)
            this._addClass(this._dropDownContentDiv, dateFilterWidgetClassNames.mobile);
        var createCalendar1 = (withButton) => {
            this._calendarWidgetsFrom = this._createDatePicker(this._dropDownContentDiv, (value) => { this._startDate = value; }, withButton);
        };
        var createCalendar2 = (withButton) => {
            this._calendarWidgetsTo = this._createDatePicker(this._dropDownContentDiv, (value) => { this._endDate = value; }, withButton);
        };
        if (filterType === 'Between') {
            createCalendar1(false);
            createCalendar2(true);
        }
        else if (filterType === 'Before')
            createCalendar2(true);
        else
            createCalendar1(true);
    }
    _createDatePicker(containerDiv, setDate, createButton) {
        if (this._options.mobileLayout)
            return this._createMobileCalendar(containerDiv, setDate, createButton);
        return this._createDesktopCalendar(containerDiv, createButton);
    }
    _createDesktopCalendar(containerDiv, createButton) {
        var div = this._createDiv(containerDiv, dateFilterWidgetClassNames.overlayCalendarContainer);
        var calendar = this._createCalendar(div, (e) => {
            dateBox.option('value', e.value);
        });
        var textBoxValueChanged = (value) => {
            calendar.option('value', value);
        };
        var dateBox = this._createTextBox(div, textBoxValueChanged);
        var submitButton = createButton ? this._createSubmitButton(div) : null;
        return new PopupCalendarWidgets(calendar, dateBox, submitButton);
    }
    _createMobileCalendar(containerDiv, setDate, createButton) {
        var div = this._createDiv(containerDiv, dateFilterWidgetClassNames.overlayCalendarContainer);
        var textBoxValueChanged = (value) => {
            setDate(value);
        };
        var dateBox = this._createTextBox(div, textBoxValueChanged);
        var submitButton = createButton ? this._createSubmitButton(div) : null;
        return new PopupCalendarWidgets(dateBox, submitButton);
    }
    _createCalendar(containerDiv, valueChanged) {
        var div = this._createDiv(containerDiv, dateFilterWidgetClassNames.overlayCalendar);
        return new dxCalendar(div, {
            focusStateEnabled: true,
            maxZoomLevel: CalendarHelper.getCalendarMaxZoomLevel(this._options.groupInterval),
            onValueChanged: valueChanged
        });
    }
    _createTextBox(containerDiv, textBoxValueChanged) {
        var div = this._createDiv(containerDiv, dateFilterWidgetClassNames.overlayTextBox);
        return new dxDateBox(div, {
            width: 'auto',
            useMaskBehavior: true,
            displayFormat: 'shortdate',
            showDropDownButton: false,
            type: 'date',
            onValueChanged: (e) => {
                textBoxValueChanged(e.value);
            }
        });
    }
    _createSubmitButton(containerDiv) {
        var div = this._createDiv(containerDiv, dateFilterWidgetClassNames.overlayButton);
        return new dxButton(div, {
            text: getLocalizationById('DashboardStringId.ButtonApply'),
            elementAttr: {
                tabindex: '0'
            },
            onClick: (e) => {
                this._submit();
            }
        });
    }
    _processItemClick(values) {
        this._options.valueChanged && this._options.valueChanged(values);
    }
    _updateDropDownButtonText() {
        var start = this._startDate;
        var end = this._endDate;
        if (this.datePickerButton) {
            if (start || end) {
                var formatHandler = this._options.format;
                var startText = start ? formatHandler(start) : null;
                var endText = end ? formatHandler(end) : null;
                var formatPatten = this._options.displayTextPattern;
                if (!formatPatten) {
                    this._setDatePickerText(constructIntervalFilterText({ left: startText, right: endText }), true);
                }
                else
                    this._setDatePickerText(stringFormat(formatPatten, startText, endText), true);
            }
            else {
                this._setDatePickerText(this._defaultButtonText, false);
            }
        }
    }
    _getDefaultOptions() {
        return {
            arrangementMode: 'AutoHeight',
            filterType: 'Between',
            datePickerLocation: 'Far',
            buttonsInfo: [],
            displayTextPattern: getLocalizationById('DashboardStringId.FromToDatePeriodCaption'),
            groupInterval: 'DayMonthYear',
            format: (value) => value.toString()
        };
    }
    _addClass(el, className) {
        el.classList.add(className);
    }
    _removeClass(el, className) {
        el.classList.remove(className);
    }
    _dispose() {
        if (this._calendarWidgetsFrom) {
            this._calendarWidgetsFrom.dispose();
            this._calendarWidgetsFrom = null;
        }
        if (this._calendarWidgetsTo) {
            this._calendarWidgetsTo.dispose();
            this._calendarWidgetsTo = null;
        }
        if (this._scrollableContent)
            this._scrollableContent.innerHTML = '';
        if (this.datePickerButton != null) {
            this.datePickerButton.dispose();
            this.datePickerButton = null;
        }
        for (var i = 0; i < this.quickButtons.length; i++)
            this.quickButtons[i].dispose();
        this.quickButtons = [];
        if (this._overlay) {
            this._overlay?.dispose && this._overlay.dispose();
            this._overlay = null;
        }
    }
    _disposeWithElement() {
        this._dispose();
        if (this._parent && this._parent.parentElement)
            this._parent.parentElement.removeChild(this._parent);
        this._parent = null;
        this._dropDownContentDiv = null;
        this._datePickerDiv = null;
        this._widgetDiv = null;
        this._scrollableContent = null;
    }
    dispose() {
        this._disposeWithElement();
    }
    _getOverlayOptions() {
        var that = this;
        const options = {
            width: this._getOverlayWidth(),
            height: 'auto',
            position: {
                collision: 'flipfit',
                my: 'top left',
                at: 'bottom left',
                of: that._datePickerDiv,
                boundary: that.boundaryElementContainer
            },
            onHidden: (e) => {
                that._overlayShown = false;
            },
            contentTemplate: (contentElement) => {
                $unwrap(contentElement).appendChild(this._dropDownContentDiv);
            },
            hideOnOutsideClick: true,
            wrapperAttr: {
                class: dateFilterWidgetClassNames.overlayWrapper
            }
        };
        options.container = that.boundaryElementContainer;
        options.animation = false;
        return options;
    }
    _getOverlayWidth() {
        if (this._options.mobileLayout)
            return function () { return getWidth(window) * 0.9; };
        return 'auto';
    }
    _setDatePickerText(text, showDropDown) {
        if (!showDropDown)
            this._addClass(this._datePickerDiv, dateFilterWidgetClassNames.datePickerButtonEmpty);
        else
            this._removeClass(this._datePickerDiv, dateFilterWidgetClassNames.datePickerButtonEmpty);
        this._datePickerContent.showDropDown = showDropDown;
        this._datePickerContent.text = text;
        this.datePickerButton.option('items', this._datePickerContent.getButtons());
    }
    _selectQuickButton(button) {
        this._performWithLockedEvents(() => {
            this._setDatePickerBtnState(false);
            for (var i = 0; i < this.quickButtons.length; i++) {
                if (button != this.quickButtons[i])
                    this.quickButtons[i].option('selectedItems', []);
            }
        });
    }
    _focusElement(element) {
        if (element.hasAttribute('tabindex') && element.getAttribute('tabindex') === '0') {
            element.focus();
        }
        else {
            var childrenWithTabIndexZero = element.querySelectorAll('[tabindex="0"]');
            if (childrenWithTabIndexZero.length > 0)
                childrenWithTabIndexZero[0].focus();
        }
    }
    _showPopup() {
        this._overlayShown = !this._overlayShown;
        this._overlay.toggle(this._overlayShown);
        fx.off = true;
        this._calendarWidgetsFrom && this._calendarWidgetsFrom.calendarWidget.option('value', this._getStartCalendarValue());
        this._calendarWidgetsTo && this._calendarWidgetsTo.calendarWidget.option('value', this._getEndCalendarValue());
        fx.off = false;
        this._removeOverlayEventListener();
        let overlay = $unwrap(this._overlay.content());
        if (overlay) {
            overlay.addEventListener('keydown', this._handleKeyDown);
            this._focusElement(overlay);
        }
    }
    _setDatePickerBtnState(checked) {
        if (this.datePickerButton) {
            if (checked && this.datePickerButton.option('selectedItemKeys')[0] !== 'Check') {
                this.datePickerButton.option('selectedItemKeys', ['Check']);
            }
            else if (!checked && this.datePickerButton.option('selectedItemKeys').length !== 0) {
                this.datePickerButton.option('selectedItemKeys', []);
            }
        }
    }
    _onDatePickerUnchecked() {
        if (this.quickButtons.every(button => button.option('selectedItemKeys').length === 0))
            this._options.clearAction();
    }
    _performWithLockedEvents(action) {
        this._lockSelectionEvents = true;
        try {
            action();
        }
        finally {
            this._lockSelectionEvents = false;
        }
    }
}
