﻿/**
* DevExpress Dashboard (_popup-menu-creator.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { $unwrap } from '@devexpress/analytics-core/analytics-internal';
import dxList from 'devextreme/ui/list';
import dxPopover from 'devextreme/ui/popover';
import 'devextreme/ui/tile_view';
import dxTileView from 'devextreme/ui/tile_view';
import { getTemplate } from '../../../data/_utils';
import { getControlContainer } from '../../viewer-items/_utils';
import { cssClasses } from './_caption-toolbar-css-classes';
export class PopupMenuCreator {
    static toggleMenu(element, menu, container, controlContainer, onMenuItemClick) {
        let popoverOptions = this._createPopoverOptions(element, menu, container, controlContainer);
        let onItemClick = (data) => {
            popover.hide();
            if (onMenuItemClick) {
                onMenuItemClick();
            }
            if (menu.itemClick)
                menu.itemClick(data.itemData, $unwrap(data.itemElement), data.itemIndex);
        };
        if (menu.type === 'icons') {
            popoverOptions.contentTemplate = (contentElement) => {
                let div = document.createElement('div');
                new dxTileView(div, this._createTileViewOptions(menu, onItemClick));
                div.setAttribute('aria-role', 'tooltip');
                div.setAttribute('aria-label', menu.title);
                return div;
            };
        }
        else {
            popoverOptions.contentTemplate = (contentElement) => {
                return new dxList(document.createElement('div'), this._createListOptions(menu, onItemClick)).element();
            };
        }
        let popoverClass = menu.type === 'icons' ? cssClasses.popoverIconMenuWrapper : cssClasses.popoverListWrapper;
        popoverOptions.wrapperAttr = { class: popoverClass };
        let popoverContainer = this._getPopupContainer(element, 'dx-dashboard-popover-marker');
        var popover = dxPopover.getInstance(popoverContainer);
        if (popover) {
            popover.option('visible') ? popover.hide() : popover.show();
            popover.option(popoverOptions);
        }
        else {
            popover = new dxPopover(popoverContainer, popoverOptions);
            popover.show();
        }
    }
    static _createPopoverOptions(element, menu, container, controlContainer) {
        let focusableParent = PopupMenuCreator.getParentWithTabIndex(element);
        if (!focusableParent)
            focusableParent = element;
        let div = document.createElement('div');
        focusableParent.insertAdjacentElement('afterend', div);
        return {
            width: 'auto',
            height: 'auto',
            target: element,
            toolbarItems: menu.type === 'icons' && menu.title ? [{ location: 'center', text: menu.title }] : [],
            animation: {
                show: { type: 'pop', from: { opacity: 1, scale: 0 }, to: { scale: 1 } },
                hide: { type: 'pop', from: { scale: 1 }, to: { scale: 0 } }
            },
            position: {
                my: 'top',
                at: 'bottom',
                collision: 'fit flip',
                of: focusableParent,
                boundary: controlContainer
            },
            container: container,
            onShown: (e) => {
                if (e.component.option('visible')) {
                    const element = $unwrap(e.component.content());
                    let popoverContent = element && element.querySelector('[tabindex]:not([tabindex="-1"]');
                    popoverContent && popoverContent.focus();
                }
            },
            onHidden(e) {
                const target = PopupMenuCreator.getParentWithTabIndex(e.component.option('target'));
                if (target) {
                    target.focus();
                }
            },
        };
    }
    static _createTileViewOptions(menu, onItemClick) {
        let itemsCount = menu.items.length;
        let columnCount = menu.columnCount ? Math.min(menu.columnCount, itemsCount) : itemsCount;
        return {
            direction: 'horizontal',
            dataSource: menu.items,
            height: this._icon_menu_element_size * Math.ceil(itemsCount / columnCount),
            itemMargin: 0,
            baseItemHeight: this._icon_menu_element_size,
            baseItemWidth: this._icon_menu_element_size,
            itemTemplate: menu.itemTemplate ? menu.itemTemplate : (itemData, itemIndex, itemElement) => {
                return getTemplate(menu.items[itemIndex]);
            },
            onItemClick: onItemClick
        };
    }
    static _createListOptions(menu, onItemClick) {
        let listOptions = {
            selectionMode: menu.selectionMode || 'none',
            selectedItems: menu.selectedItems || [],
            dataSource: menu.items,
            onItemClick: onItemClick
        };
        if (menu.itemTemplate) {
            listOptions.itemTemplate = menu.itemTemplate;
        }
        return listOptions;
    }
    static _getPopupContainer(element, markerClassName) {
        let popupContainer = element.querySelectorAll('.' + markerClassName);
        if (popupContainer.length > 1) {
            console.log('Multiple popovers');
        }
        else if (popupContainer.length === 0) {
            let div = document.createElement('div');
            div.classList.add(markerClassName);
            element.appendChild(div);
            return div;
        }
        else {
            return popupContainer[0];
        }
    }
    static getParentWithTabIndex(element) {
        const controlContainer = getControlContainer(element);
        if (!element || !element.parentElement || element === controlContainer)
            return null;
        const parent = element.parentElement;
        if (parent?.hasAttribute('tabindex')) {
            return parent;
        }
        else {
            return PopupMenuCreator.getParentWithTabIndex(parent);
        }
    }
}
PopupMenuCreator._icon_menu_element_size = 65;
