﻿/**
* DevExpress Dashboard (_parameters-dialog.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import DataSource from 'devextreme/data/data_source';
import 'devextreme/ui/check_box';
import dxCheckBox from 'devextreme/ui/check_box';
import dxDataGrid from 'devextreme/ui/data_grid';
import dxDateBox from 'devextreme/ui/date_box';
import dxLoadPanel from 'devextreme/ui/load_panel';
import dxNumberBox from 'devextreme/ui/number_box';
import dxSelectBox from 'devextreme/ui/select_box';
import dxTagBox from 'devextreme/ui/tag_box';
import dxTextBox from 'devextreme/ui/text_box';
import { current as currentTheme, isMaterial } from 'devextreme/ui/themes';
import * as $ from 'jquery';
import { $unwrap, createJQueryCallbacks } from '../../../data/_jquery-helpers';
import { localizationId } from '../../../data/_localization-ids';
import { localizer } from '../../../data/_localizer';
import { getLocalizationById } from '../../../data/localization/_default';
import { ParameterHelper } from '../../../model/parameters/_parameters-helper';
import { dashboardConstants, devExtremeConstants } from '../../../styles/_constants';
import { DashboardLayoutModeHelper } from '../../_dashboard-layout-mode-helper';
import { dialogForm, dialogClasses as dialogFormClasses, dialogSizes } from './_dialog-form';
const genericParametersDialogSizes = {
    fakeElementsHeight: 172,
    fakeParametersDialogGridRowHeight: 34,
};
var dialogClasses = {
    allowNullCheckBox: 'dx-parameter-allownull-checkbox',
    allowNullCheckBoxSize: 'dx-datagrid-checkbox-size',
    valueEditor: 'dx-parameter-value-editor',
    multiselectValuePart: 'dx-dashboard-dialog-parameters-tag',
    theme: 'dx-dashboard-theme',
};
export let parameterTypes = {
    string: 'String',
    int: 'Int',
    float: 'Float',
    bool: 'Bool',
    dateTime: 'DateTime',
    selector: 'Selector',
    multiselector: 'Multiselector',
    guid: 'Guid'
};
export class parametersDialog {
    constructor(options) {
        this.valueChanged = createJQueryCallbacks();
        this.parameterEntitiesDict = {};
        this.options = options;
        if (this.options.getParametersCollection) {
            this.getParametersCollection = this.options.getParametersCollection;
        }
        if (this.options.submitParameters) {
            this.submitParameters = this.options.submitParameters;
        }
        this._initialize();
    }
    _initialize() {
        var that = this, options = that.options, numberOfParameters = that.getParametersCollection().getVisibleParameters().length, scroll = numberOfParameters > 8, allowNullColumn = that.allowNullColumn(), submitParameters = that.submitParameters, parameterEntities = [];
        let notScrollHeight = isMaterial(currentTheme()) ?
            devExtremeConstants.materialPopupToolbarHeight + devExtremeConstants.dxPopupTitleBorderBottom + 2 * devExtremeConstants.dxDatagridHeadersBorder +
                dashboardConstants.materialParametersDialogFormPaddingTop +
                devExtremeConstants.materialGridBaseHeaderHeight + devExtremeConstants.dxDatagridHeadersBorder +
                numberOfParameters * (devExtremeConstants.materialGridBaseCellHeight + devExtremeConstants.materialGridBaseRowBorder) +
                dashboardConstants.materialParametersDialogFormPaddingBottom +
                devExtremeConstants.materialPopupContentPadding * 2 + devExtremeConstants.materialButtonHeight
            :
                (numberOfParameters + 1) * genericParametersDialogSizes.fakeParametersDialogGridRowHeight + genericParametersDialogSizes.fakeElementsHeight;
        that.dialogForm = new dialogForm({
            title: localizer.getString(localizationId.labelName.ParametersFormCaption),
            fullScreenMode: options.fullScreenMode,
            dialogContainer: options.parametersDialogContainer,
            width: allowNullColumn ? dialogSizes.width : dialogSizes.minWidth,
            height: scroll ? dialogSizes.height : notScrollHeight,
            allowScrolling: false,
            deferredRendering: false,
            onShowing: options.onShowing,
            onShown: options.onShown,
            onHidden: (args) => {
                options.onHidden(args);
                this._loadPanel?.hide();
            },
            buttons: [{
                    name: localizer.getString(localizationId.buttonNames.ButtonReset), func: () => that.resetParameterValues()
                }, {
                    name: localizer.getString(localizationId.buttonNames.ButtonSubmit), hide: true, func: () => that.submitParameterValues(), isDefault: true
                }, {
                    name: localizer.getString(localizationId.buttonNames.ButtonCancel), hide: true, func: () => { }
                }],
            renderContent: (controlCreationCallbacks) => {
                var parametersForm = document.createElement('div');
                parametersForm.classList.add(dialogFormClasses.form);
                this._dataGrid = that._generateContent(parametersForm, controlCreationCallbacks, true);
                return parametersForm;
            },
            disposeContent: () => {
                this._disposeGrid();
            },
            setActualState: function () {
                that.setActualState();
            }
        });
        let loadingPanelElement = document.createElement('div');
        options.parametersDialogContainer.appendChild(loadingPanelElement);
        this._loadPanel = new dxLoadPanel(loadingPanelElement, {
            shadingColor: 'rgba(0,0,0,0.4)',
            position: { of: $unwrap(this.dialogForm.popupInstance.content()) },
            visible: false,
            showIndicator: true,
            showPane: true,
            shading: false,
            hideOnOutsideClick: false,
        });
    }
    showLoadPanel() {
        this._loadPanel?.show();
    }
    hideLoadPanel() {
        this._loadPanel?.hide();
    }
    _disposeGrid() {
        if (this._dataGrid) {
            this._dataGrid.option('dataSource').forEach(entry => entry.dispose());
            this._dataGrid.dispose();
        }
    }
    appendNullGridColumn(gridColumns) {
        if (this.allowNullColumn())
            gridColumns.push(this.createNullColumn());
    }
    allowNullColumn() {
        var allowNullValues;
        this.getParametersCollection().getVisibleParameters().forEach(parameter => {
            if (parameter.getAllowNull())
                allowNullValues = true;
        });
        return allowNullValues;
    }
    createNullColumn() {
        return {
            dataField: 'fakeDataField_divAllowNull',
            caption: localizer.getString(localizationId.labelName.ParametersFormAllowNullColumnCaption),
            width: '20%',
            alignment: 'center',
            allowEditing: false,
            cellTemplate: function (container, options) {
                let entity = options.data;
                $unwrap(container).appendChild($unwrap(entity.divAllowNull));
            }
        };
    }
    createGridColumns() {
        let allowNullColumn = this.allowNullColumn();
        let gridColumns = [{
                dataField: 'description',
                caption: localizer.getString(localizationId.labelName.ParametersFormNameColumnCaption),
                dataType: 'string',
                width: allowNullColumn ? '40%' : '50%',
                allowEditing: false
            }, {
                dataField: 'fakeDataField_divValueEditor',
                caption: localizer.getString(localizationId.labelName.ParametersFormValueColumnCaption),
                width: allowNullColumn ? '40%' : '50%',
                cssClass: 'dx-parameter-value-editor',
                showEditorAlways: true,
                editCellTemplate: (cellElement, cellInfo) => {
                    $unwrap(cellElement).appendChild($unwrap(cellInfo.data.divValueEditor));
                }
            }];
        this.appendNullGridColumn(gridColumns);
        return gridColumns;
    }
    _generateContent(element, controlCreationCallbacks, subscribeValueChanged = false) {
        var that = this, parameterEntities = that.getParametersCollection().getParameterList().reduce((acc, parameter) => {
            let parameterEntity = that._getParameterEntity(parameter, controlCreationCallbacks);
            if (parameterEntity) {
                if (subscribeValueChanged) {
                    parameterEntity.valueChanged.add((e) => that.valueChanged.fire());
                }
                acc.push(parameterEntity);
            }
            return acc;
        }, []);
        this.submitParameterValues = () => {
            that.submitParameters(parameterEntities.map(parameterEntity => parameterEntity.wrapParameter()));
        };
        this.resetParameterValues = () => {
            var parametersCollection = that.getParametersCollection();
            parameterEntities.forEach(parameterEntity => {
                parameterEntity.setValue(parametersCollection.getParameterDefaultValue(parameterEntity.name));
                this.options.updateCascadingParameters(parameterEntity.name, parameterEntity.getValue(), this.parameterEntitiesDict, true);
            });
        };
        this.setActualState = () => {
            var parametersCollection = that.getParametersCollection();
            parameterEntities.forEach(parameterEntity => {
                var parameter = parametersCollection.getParameterByName(parameterEntity.name);
                if (parameter) {
                    const converter = ParameterHelper.getTypeConverter(parameter.originalType);
                    var lookUpValues = parameter.getLookUpValues();
                    if (lookUpValues !== null) {
                        converter && lookUpValues.forEach(x => {
                            x._value = converter(x._value);
                        });
                        parameterEntity.setLookUpValues(lookUpValues);
                    }
                    parameterEntity.setValue(parameter.getValue());
                }
            });
        };
        let gridOptions = {
            dataSource: parameterEntities,
            columns: that.createGridColumns(),
            width: '100%',
            height: '100%',
            showColumnLines: true,
            showRowLines: true,
            allowColumnResizing: true,
            keyExpr: 'name',
            loadPanel: {
                enabled: false
            },
            repaintChangesOnly: true,
            editing: { mode: 'cell', allowUpdating: true, refreshMode: 'repaint' },
            paging: { enabled: false },
            sorting: { mode: 'none' },
            scrolling: {
                mode: 'standard'
            }
        };
        if (isMaterial(currentTheme())) {
            gridOptions.showBorders = true;
            gridOptions.rowAlternationEnabled = parameterEntities.length > 2;
        }
        return new dxDataGrid(element, gridOptions);
    }
    generateContent(element, disposeCallback) {
        let controlCreationCallbacks = createJQueryCallbacks();
        this._disposeGrid();
        this._dataGrid = this._generateContent(element, controlCreationCallbacks, true);
        let prepareActualValues = (controlCreationCallbacks) => {
            controlCreationCallbacks.fire();
            this.setActualState();
        };
        prepareActualValues(controlCreationCallbacks);
        return {
            grid: this._dataGrid,
            submitParameterValues: () => this.submitParameterValues(),
            resetParameterValues: () => this.resetParameterValues(),
            valueChanged: this.valueChanged,
            dispose: () => {
                this.dispose();
                this._dataGrid?.dispose && this._dataGrid?.dispose();
                disposeCallback && disposeCallback();
            }
        };
    }
    show() {
        this.dialogForm.showDialog();
    }
    hide() {
        this.dialogForm.hideDialog();
        this.parameterEntitiesDict = {};
    }
    dispose() {
        this.dialogForm && this.dialogForm.dispose();
        this.parameterEntitiesDict = {};
        this._disposeGrid();
        this._loadPanel?.dispose();
    }
    _getParameterEntity(parameter, controlCreationCallbacks) {
        var entityOptions = {
            name: parameter.getName(),
            description: parameter.getDescription(),
            defaultValue: parameter.getDefaultValue(),
            controlCreationCallbacks: controlCreationCallbacks,
            allowNull: parameter.getAllowNull(),
            allowMultiselect: parameter.getAllowMultiselect(),
            type: parameter.getType(),
            value: parameter.getValue(),
            selectAllValues: parameter._selectAllValues
        };
        let parameterEntity;
        if (parameter.getLookUpValues() !== null) {
            if (entityOptions.allowMultiselect) {
                parameterEntity = new ParameterEntity({
                    ...entityOptions,
                    valueName: 'value',
                    createControl: (element) => new dxTagBox(element, {
                        showDropDownButton: true,
                        showSelectionControls: true,
                        selectAllMode: 'allPages',
                        multiline: false,
                        tagTemplate: (data, $element) => {
                            var element = $unwrap($element);
                            if (element) {
                                element.innerText = $.fn.constructor(element).is(':first-child') ? data.displayValue : ', ' + data.displayValue;
                                element.classList.add(dialogClasses.multiselectValuePart);
                                return element;
                            }
                            return undefined;
                        },
                        searchEnabled: true,
                        selectAllText: getLocalizationById('DashboardStringId.FilterElementShowAllItem'),
                        noDataText: getLocalizationById('DashboardStringId.FilterElementNoDataToDisplay'),
                        displayExpr: 'displayValue',
                        valueExpr: 'value',
                        value: entityOptions.value,
                        searchExpr: 'displayValue',
                        placeholder: localizer.getString(localizationId.ParametersSelectorText),
                        dropDownOptions: {
                            container: this.options.parametersDialogContainer
                        },
                        onSelectionChanged: e => {
                            e.element.title = e.component.option('value').join(', ');
                        },
                        onFocusOut: (e) => {
                            if (e.component['_isVisible'] && e.component['_isVisible']() && this.options.updateCascadingParameters)
                                this.options.updateCascadingParameters(entityOptions.name, e.component.option('selectedItems').map(item => item.value), this.parameterEntitiesDict, true);
                        }
                    })
                });
            }
            else {
                parameterEntity = new ParameterEntity({
                    ...entityOptions,
                    valueName: 'value',
                    createControl: (element) => new dxSelectBox(element, {
                        searchEnabled: true,
                        selectAllText: getLocalizationById('DashboardStringId.FilterElementShowAllItem'),
                        noDataText: getLocalizationById('DashboardStringId.FilterElementNoDataToDisplay'),
                        encodeNoDataText: true,
                        displayExpr: 'displayValue',
                        valueExpr: 'value',
                        searchExpr: 'displayValue',
                        placeholder: localizer.getString(localizationId.ParametersSelectorText),
                        dropDownOptions: {
                            container: this.options.parametersDialogContainer
                        },
                        onSelectionChanged: e => {
                            if (e.component['_isVisible'] && e.component['_isVisible']() && this.options.updateCascadingParameters)
                                this.options.updateCascadingParameters(entityOptions.name, e.selectedItem?.value, this.parameterEntitiesDict, true);
                        },
                    })
                });
            }
        }
        else {
            const onFocusOut = (e, value) => {
                if (e.component['_isVisible'] && e.component['_isVisible']() && this.options.updateCascadingParameters)
                    this.options.updateCascadingParameters(entityOptions.name, value, this.parameterEntitiesDict, true);
            };
            switch (parameter.getType()) {
                case parameterTypes.string:
                    parameterEntity = new ParameterEntity({
                        ...entityOptions,
                        valueName: 'value',
                        createControl: (element) => new dxTextBox(element, {
                            onFocusOut: e => onFocusOut(e, e.component.option('value'))
                        })
                    });
                    break;
                case parameterTypes.int:
                    parameterEntity = new ParameterEntity({
                        ...entityOptions,
                        valueName: 'value',
                        createControl: (element) => new dxNumberBox(element, {
                            showSpinButtons: true,
                            step: 1,
                            onFocusOut: e => onFocusOut(e, e.component.option('value'))
                        })
                    });
                    break;
                case parameterTypes.float:
                    parameterEntity = new ParameterEntity({
                        ...entityOptions,
                        valueName: 'value',
                        createControl: (element) => new dxNumberBox(element, {
                            showSpinButtons: true,
                            step: 0.1,
                            onFocusOut: e => onFocusOut(e, e.component.option('value'))
                        })
                    });
                    break;
                case parameterTypes.bool:
                    parameterEntity = new ParameterEntity({
                        ...entityOptions,
                        valueName: 'value',
                        createControl: (element) => new dxCheckBox(element, {
                            onValueChanged: e => onFocusOut(e, e.value)
                        })
                    });
                    break;
                case parameterTypes.dateTime:
                    parameterEntity = new ParameterEntity({
                        ...entityOptions,
                        valueName: 'value',
                        createControl: (element) => new dxDateBox(element, {
                            pickerType: DashboardLayoutModeHelper.isTouch ? 'rollers' : 'calendar',
                            width: '100%',
                            applyButtonText: localizer.getString(localizationId.buttonNames.ButtonOK),
                            cancelButtonText: localizer.getString(localizationId.buttonNames.ButtonCancel),
                            placeholder: localizer.getString(localizationId.buttonNames.SelectDate),
                            dropDownOptions: {
                                container: this.options.parametersDialogContainer
                            },
                            onPopupInitialized: function (e) {
                                var popup = e.popup;
                                if (popup) {
                                    let todayBtn = {
                                        widget: 'dxButton', toolbar: 'bottom', location: 'center',
                                        options: {
                                            text: localizer.getString(localizationId.labelName.ParametersFormCalendarTodayButton),
                                            onClick: function () {
                                                var dateBox = e.component;
                                                if (dateBox) {
                                                    var todate = new Date();
                                                    todate.setHours(0, 0, 0, 0);
                                                    dateBox.option('value', todate);
                                                }
                                            }
                                        }
                                    };
                                    if (DashboardLayoutModeHelper.isTouch) {
                                        popup.option('toolbarItems').push(todayBtn);
                                    }
                                    else {
                                        popup.option('toolbarItems', [todayBtn]);
                                    }
                                }
                            },
                            onFocusOut: e => onFocusOut(e, e.component.option('value'))
                        })
                    });
                    break;
                case parameterTypes.guid:
                    parameterEntity = new ParameterEntity({
                        ...entityOptions,
                        valueName: 'value',
                        createControl: (element) => new dxTextBox(element, {
                            mask: 'hhhhhhhh-hhhh-hhhh-hhhh-hhhhhhhhhhhh',
                            maskRules: { 'h': /[0-9A-Fa-f]/ },
                            useMaskedValue: true,
                            onFocusOut: e => onFocusOut(e, e.component.option('value'))
                        })
                    });
                    break;
            }
        }
        this.parameterEntitiesDict[entityOptions.name] = parameterEntity;
        if (parameter.isVisible())
            return parameterEntity;
    }
}
export class ParameterEntity {
    dispose() {
        if (this.allowNullControl) {
            this.allowNullControl.dispose();
        }
        if (this.control) {
            this.control.dispose();
        }
        this.divValueEditor = null;
        this.divAllowNull = null;
        this.allowNullControl = null;
    }
    constructor(options) {
        this.lookUpValues = [];
        this.valueChanged = createJQueryCallbacks();
        this.name = options.name;
        this.type = options.type;
        this.description = options.description ? options.description : this.name;
        this.defaultValue = options.defaultValue;
        this.value = options.value;
        this.lookUpValues = [];
        this.allowNull = options.allowNull;
        this.allowMultiselect = options.allowMultiselect;
        this.createControl = options.createControl;
        this.valueName = options.valueName;
        this.controlCreationCallbacks = options.controlCreationCallbacks;
        this.selectAllValues = options.selectAllValues;
        this.divValueEditor = document.createElement('div');
        this.divValueEditor.classList.add(dialogClasses.valueEditor);
        this.divValueEditor.classList.add(dialogClasses.valueEditor + '-' + this.type.toLowerCase());
        this.divValueEditor.classList.add(dialogClasses.allowNullCheckBoxSize);
        if (this.allowNull) {
            this.divAllowNull = document.createElement('div');
            this.divAllowNull.classList.add(dialogClasses.allowNullCheckBox);
            this.divAllowNull.classList.add(dialogClasses.allowNullCheckBoxSize);
        }
        else {
            this.divAllowNull = document.createElement('center');
            this.divAllowNull.innerText = 'n/a';
        }
        this._addControl();
    }
    getValue() {
        if (this.allowNull && this.allowNullControl.option('value') === true)
            return null;
        else
            return this.control.option(this.valueName);
    }
    setValue(value) {
        if (this.allowMultiselect) {
            if ((value === null) || (value === undefined)) {
                value = [];
            }
            else if (!Array.isArray(value)) {
                value = [{ displayValue: value, value: value }];
            }
        }
        this.control.option('value', value);
    }
    setLookUpValues(values) {
        var newValues = [];
        values.forEach(value => {
            newValues.push({
                value: value.getValue(),
                displayValue: value.getDisplayText()
            });
        });
        this.lookUpValues = newValues;
        this.control.option('dataSource', new DataSource(newValues));
    }
    wrapParameter() {
        return {
            Name: this.name,
            Value: this.getValue()
        };
    }
    _addControl() {
        this.controlCreationCallbacks.add((component) => {
            if (!this.control) {
                this.control = this.createControl(this.divValueEditor);
                this.control.option('onValueChanged', this.allowNull ?
                    (e) => {
                        var passNull = this.allowNullControl.option('value'), value = this.control.option('value');
                        if (this.allowMultiselect) {
                            if (passNull === true && value.length > 0)
                                this.allowNullControl.option('value', false);
                            else if (passNull === false && value.length === 0)
                                this.allowNullControl.option('value', true);
                        }
                        else if (passNull === false && value === null)
                            this.allowNullControl.option('value', true);
                        else if (passNull === true && value !== null)
                            this.allowNullControl.option('value', false);
                        this.valueChanged.fire(e);
                    } :
                    (e) => {
                        this.valueChanged.fire(e);
                    });
                if (this.allowNull) {
                    this.allowNullControl = new dxCheckBox(this.divAllowNull, {
                        value: this.value === null,
                        onValueChanged: (e) => {
                            var value = this.control.option('value');
                            if (e.value) {
                                this.value = value;
                                if (this.allowMultiselect)
                                    this.control.option('value', []);
                                else if (value !== null)
                                    this.control.option('value', null);
                            }
                            else if (value === null || value.length === 0)
                                this.control.option('value', this.value);
                        }
                    });
                }
            }
        });
    }
}
