﻿/**
* DevExpress Dashboard (_arrangement-strategy.js)
* Version:  24.2.7
* Build date: Apr 29, 2025
* Copyright (c) 2012 - 2025 Developer Express Inc. ALL RIGHTS RESERVED
* License: https://www.devexpress.com/Support/EULAs/universal.xml
*/
import { borders, cardArrangementInfo, margins, paddings } from './_card-arrangement-info';
import { cardMeasurer } from './_card-measurer';
export class ArrangementStrategy {
    static createInstance(method, lineCount) {
        switch (method) {
            case 'column':
                return new ColumnArrangementStrategy(lineCount);
            case 'row':
                return new RowArrangementStrategy(lineCount);
            default:
                return new AutoArrangementStrategy();
        }
    }
    getArrangeMethod() {
        return '';
    }
    getLineCount() {
        return -1;
    }
    constructor() {
        this.cardMeasurer = new cardMeasurer();
    }
    arrange(viewerHeight, viewerWidth, itemsCount, layouts, ignorePadding = false) {
        var getArrangementInfo = (nextColumnCount = undefined) => {
            var result = new cardArrangementInfo();
            var columnCount = nextColumnCount || this.getColumnCount(itemsCount, actualViewerWidth, cardMinWidth);
            columnCount = this.checkEmptyRows(columnCount, itemsCount);
            var rowCount = this.getRowCount(itemsCount, columnCount);
            var visibleColumnCount = Math.max(1, Math.min(columnCount, Math.floor(actualViewerWidth / cardMinWidth)));
            var visibleRowCount = Math.max(1, Math.min(rowCount, Math.floor(actualViewerHeight / cardMinHeight)));
            result.cellHeight = Math.max(cardMinHeight, Math.floor(actualViewerHeight / visibleRowCount));
            result.cellWidth = Math.max(cardMinWidth, Math.floor(actualViewerWidth / visibleColumnCount));
            result.columnCount = columnCount;
            result.rowCount = rowCount;
            result.itemsCount = itemsCount;
            result.ignorePadding = ignorePadding;
            result.viewerHeight = viewerHeight;
            result.viewerWidth = viewerWidth;
            result.arrangementDirection = this.getArrangementDirection();
            return result;
        };
        var marginOffset = ignorePadding ? margins : 0;
        var cardMinWidth = this.getCardMinWidthWithMargin(layouts);
        var cardMinHeight = this.getCardMinHeightWithMargin(layouts);
        var actualViewerWidth = viewerWidth + marginOffset;
        var actualViewerHeight = viewerHeight + marginOffset;
        return this.calcBestProportions(getArrangementInfo);
    }
    calcBestProportions(calcArrangementInfo) {
        return calcArrangementInfo();
    }
    getCardMinWidthWithMargin(layouts) {
        var largestWidth = layouts.map(layout => layout.minWidth).reduce((prev, curr) => Math.max(prev, curr));
        return largestWidth + paddings + borders + margins;
    }
    getCardMinHeightWithMargin(layouts) {
        var largestHeight = layouts.map(layout => this.cardMeasurer.calcMinHeight(layout)).reduce((prev, curr) => Math.max(prev, curr));
        return largestHeight + paddings + borders + margins;
    }
    checkEmptyRows(columnCount, itemsCount) {
        return columnCount;
    }
    getArrangementDirection() {
        return 'column';
    }
}
export class AutoArrangementStrategy extends ArrangementStrategy {
    constructor() {
        super();
    }
    getArrangeMethod() {
        return 'auto';
    }
    getLineCount() {
        return -1;
    }
    getColumnCount(itemsCount, actualViewerWidth, cardMinWidth) {
        var columnCount = Math.max(1, Math.floor(actualViewerWidth / cardMinWidth));
        columnCount = Math.min(itemsCount, columnCount);
        return columnCount;
    }
    checkEmptyRows(columnCount, itemsCount) {
        var lastRowCount = () => itemsCount % columnCount;
        var fullRowCount = () => Math.floor(itemsCount / columnCount);
        var nextLastRowCount = () => lastRowCount() + fullRowCount();
        while (lastRowCount() > 0 && (nextLastRowCount() < columnCount)) {
            columnCount--;
        }
        return columnCount;
    }
    calcBestProportions(calcArrangementInfo) {
        var arrangementInfo = calcArrangementInfo();
        if (arrangementInfo.columnCount > 1) {
            var nextArrangementInfo = calcArrangementInfo(arrangementInfo.columnCount - 1);
            while (nextArrangementInfo.proportionFactor() < arrangementInfo.proportionFactor() && arrangementInfo.columnCount > 1) {
                arrangementInfo = nextArrangementInfo;
                nextArrangementInfo = calcArrangementInfo(arrangementInfo.columnCount - 1);
            }
        }
        return arrangementInfo;
    }
    getRowCount(itemsCount, columnCount) {
        return Math.ceil(itemsCount / columnCount);
    }
}
export class ColumnArrangementStrategy extends ArrangementStrategy {
    constructor(columnCount) {
        super();
        this.columnCount = columnCount;
    }
    getArrangeMethod() {
        return 'column';
    }
    getLineCount() {
        return this.columnCount;
    }
    getColumnCount(itemsCount, actualViewerWidth, cardMinWidth) {
        return Math.min(itemsCount, this.columnCount);
    }
    getRowCount(itemsCount, columnCount) {
        return Math.max(1, Math.ceil(itemsCount / columnCount));
    }
}
export class RowArrangementStrategy extends ArrangementStrategy {
    constructor(rowCount) {
        super();
        this.rowCount = rowCount;
    }
    getArrangementMethod() {
        return 'row';
    }
    getLineCount() {
        return this.rowCount;
    }
    getColumnCount(itemsCount, actualViewerWidth, cardMinWidth) {
        return Math.ceil(itemsCount / Math.min(itemsCount, this.rowCount));
    }
    getRowCount(itemsCount, columnCount) {
        return Math.min(itemsCount, this.rowCount);
    }
    getArrangementDirection() {
        return 'row';
    }
}
